#
# This file is part of Config-Model
#
# This software is Copyright (c) 2014 by Dominique Dumont.
#
# This is free software, licensed under:
#
#   The GNU Lesser General Public License, Version 2.1, February 1999
#
{
	set_by_user => 'user value',
	set_in_both => 'user value2',
	a_checklist => {qw/e 0 f 1 g 1/}
};
